#!/usr/bin/perl -w

####################################################
# this script reads a protein databank file and
# generates a list of atoms for a feff.inp file
# make this file an executable by typing "chmod 777 pdb2fi.pl"
# type "pdb2fi.pl filename.pdb" to run the program
#################################################
my($central_atom) = "FE";

#define ipot numbers
%IPOT = (
	 "CORE" => 0,
	 "O" => 1,
	 "C" => 2,
	 "N" => 3,
	 "FE"=> 4,
	 "S" => 5);
#You should not have to change anything past this line
############################################################
my($cnum) = 0;   #the first occurance of Fe in the pdb file
my($i) = 0;
my($line);
my(@atoms);   #holds atom list from protein databank file
my($cindex);  #holds index of central atom
my ($filename) = $ARGV[0];
my(@atom,@x,@y,@z,@tag,@dis);  #holds information for output
my(%LIST);                   #order of atoms by distance 

#read atom list from pdb file
print "filename is $filename\n";
open(INPUT, $filename) || die "can not open $filename";
while($line = <INPUT>){
  if ($line =~ /^ATOM|^HETATM/){
    $line =~ s/^HETATM/HETATM /;
    $atoms[$i++] = $line;
    #print "$line\n";
  }
}
close (INPUT);

#get important info from atoms array
$i = 0;
foreach $atoms (@atoms){
  $atoms =~ s/[\s]+$|\n//g;
#  $atoms =~ / (\w+)$/;  #last word is atom type
#  $atom[$i] = $1;
  $atoms =~ /\d+\s+(\w+)/;
  $atom[$i] = $1;
#  print "atom = $atom[$i]\n";
  #xyz coordinates are always followed by 5 spaces
  @parts = split(/^.{31}/,$atoms);  #coordinates always follow 31 characters
  $parts[1] =~ s/^[\s]+//g;
  @words1 = split(/[\s]+/,$parts[1]);
  $x[$i] = $words1[0];      #x coordinate
  $y[$i] = $words1[1];      #y coordinate
  $z[$i] = $words1[2];      #z coordinate
  #  print "word3=$words[3], lastword=$words[@words]\n";
  $atoms =~ / (\w\w\w) /;
  $tag = $1;
  $tag[$i] = $tag . "_" . $atom[$i];  #tag
  #print "atom = $atom[$i], x = $x[$i], y = $y[$i], z = $z[$i] tag = $tag[$i] \n";
  $i++;
}

#find central atom
$j = 0;
for ($i=0; $i<@atom; $i++){
  #print "atom=$atom[$i]\n";
  if ($atom[$i] =~/$central_atom/){
    print "found central atom $central_atom at $atom[$i] $x[$i], $y[$i],$z[$i] \n";
    $cx[$j] = $x[$i];
    $cy[$j] = $y[$i];
    $cz[$j++] = $z[$i];
  }
}


 

#shift coordinates so centeral atom is at zero
#and determine distance
#throw away all atoms that are more than 10 angstroms from central atom
print "using central atom at $cx[$cnum],$cy[$cnum],$cz[$cnum]\n";
$j = 0;
for ($i=0; $i<@atom; $i++){
#  print "$x[$i],$y[$i],$z[$i]\n";
  $x[$i] = $x[$i] - $cx[$cnum];
  $y[$i] = $y[$i] - $cy[$cnum];
  $z[$i] = $z[$i] - $cz[$cnum];
#  print "$x[$i],$y[$i],$z[$i]\n";
  $distance = sqrt(($x[$i]*$x[$i])+($y[$i]*$y[$i])+($z[$i]*$z[$i]));
#  print "distance = $distance\n";
  if ($distance < 10){
    $dis[$j] = $distance;
    $LIST{$dis[$j++]} = $i;  #put index into a hash to keep track of sorted order
  }
}

#sort distances
for ($i=0; $i<@dis; $i++){
  next if !(defined($dis[$i]));
  for ($j=$i+1; $j<@dis; $j++){
    next if !(defined($dis[$j]));
    #print "$i $dis[$i], $j $dis[$j]\n";
    if($dis[$j] < $dis[$i]){
      $temp = $dis[$i];
      $dis[$i] = $dis[$j];
      $dis[$j] = $temp;
    }
  }
}

#open output file and write stuff
$out = ">" . "$filename" . "_feff.inp";
open (OUT,$out) || die "can not open output\n";
print "writing file $out\n";
$atom[$LIST{0}] = "CORE";  #get ipot correct for core atom
for ($i=0; $i<@dis; $i++){
  $j = $LIST{$dis[$i]};
  printf OUT "%10.5f%10.5f%10.5f%10d%10s%10.5f\n",
    $x[$j], $y[$j], $z[$j], "1", $tag[$j], $dis[$i];
}

close(OUT);
  
